/*****************************************************************************
* Copyright:    (c) 2024 AMFELTEC Corp.
* ----------------------------------------------------------------------------
This file is part of AMFELTEC System Logger Software Package

# AMFELTEC System Logger Software Package 
# is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AMFELTEC System Logger Software Package
# is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <https://www.gnu.org/licenses/>

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/time.h>

#include <sys/stat.h>
#include <sys/types.h>

#include "comport.h"

int SerialPort;

int close_port(void) {
   return close(SerialPort); 
}

char* RetreivePortName(void){
    char *port_path = NULL;
    port_path = getenv("TTYUSB_PORT");
    if (port_path == NULL)
        return NULL;
    return port_path;
}

int open_serial_port(char *port) {
    struct termios tty;

    SerialPort = open(port, O_RDWR | O_NOCTTY);
    if (SerialPort == -1) {
        perror("Error opening serial port");
        return EXIT_FAILURE;
    }

    memset(&tty, 0, sizeof(tty));
    if (tcgetattr(SerialPort, &tty) != 0) {
        perror("Error getting serial port attributes");
        close(SerialPort);
        return EXIT_FAILURE;
    }

    tty.c_iflag &= IGNCR;
    tty.c_oflag = OCRNL; 
    tty.c_cflag |= (CLOCAL | CREAD);
    tty.c_cflag &= ~PARENB;
    tty.c_cflag &= ~CSTOPB; 
    tty.c_cflag &= ~CSIZE;
    tty.c_cflag |= CS8;

    tty.c_lflag &= ~(ICANON | ECHOE | ECHO | ISIG);

    cfsetospeed(&tty, B57600);
    cfsetispeed(&tty, B57600);

    if (tcsetattr(SerialPort, TCSANOW, &tty) != 0) {
        perror("Error setting serial port attributes");
        close(SerialPort);
        return 1;
    }
    return EXIT_SUCCESS;
}

int GetSerialPort() {
    return SerialPort;
}

void write_to_serial(int serialPort, unsigned char bytesToWrite[], int num){
    if (num != -1) {
        unsigned char chToWrite = bytesToWrite[num];
        if (write(serialPort, &chToWrite, 1) == -1)
        {            
            printf("Can't write %hhu to port %d\n", chToWrite, serialPort);
        } 
#ifdef DEBUG        
        else {
            printf("Wrote %hhu to port %d\n", chToWrite, serialPort);
        }
#endif
    } else {
        if (write(serialPort, (const char *)bytesToWrite, strlen((const char *)bytesToWrite)) == -1)
        {
            printf("Can't write %s to port %d\n", bytesToWrite, serialPort);
        } 
#ifdef DEBUG        
        else {
            printf("Wrote %s to port %d\n", bytesToWrite, serialPort);
        }
#endif        
    }
}

fd_set readSet;
struct timeval timeout; 
int comPortAvailable = -10;

int PortComIsReady(int sec, int usec) {

    int comPort = GetSerialPort();
    FD_ZERO(&readSet);
    FD_SET(comPort, &readSet);

    timeout.tv_sec = sec;
    timeout.tv_usec = usec;
    comPortAvailable = select(comPort + 1, &readSet, NULL, NULL, &timeout);
    return comPortAvailable;
}

int GetComPortStatus(void){
    return comPortAvailable;
}

void read_info_from_port(int serialPort) {
    char buffer[512];
    char buff[2];
    int bytesRead;
    int totalBytesRead = 0;
    buffer[0] = '\0';

    do {    

        PortComIsReady(1, 0);
        if (GetComPortStatus() > 0) { // can read here

            buff[0] = '\0';                            
            bytesRead = read(serialPort, buff, 1);
            if (bytesRead < 0) {
                perror("Error reading from port - trace 1\n");
            }
            buffer[totalBytesRead++] = buff[0]; 

        } else if (GetComPortStatus() == 0) {  // 1 sec timeout 
//#ifdef DEBUG        
            printf("No data received from com port within 2 seconds\n");
//#endif
//            ;
        } else { // reading error
            perror("Error reading from port - trace 2\n");
        }
    } while (GetComPortStatus() > 0); 
    buffer[totalBytesRead + 1] = '\0';
//#ifdef DEBUG    
    printf("Finished reading, buffer size is [%ld], total bytes read [%d]\n", strlen(buffer), totalBytesRead);   
//#endif    
    printf("%s\n", buffer);
}
