/*****************************************************************************
* Copyright:    (c) 2024 AMFELTEC Corp.
* ----------------------------------------------------------------------------
This file is part of AMFELTEC System Logger Software Package

# AMFELTEC System Logger Software Package 
# is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AMFELTEC System Logger Software Package
# is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <https://www.gnu.org/licenses/>

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comport.h"
#include "parser.h"
#include "outpututils.h"
#include "configfile.h"

void usage(void) 
{
    printf("Usage :\n");
    printf("./logger [command] <parameters>\n");
    printf("List of valid commands & parameters:\n");
//    printf(" - port <ttyUSB[0-9]>\n");
    printf(" - set \n");
    printf(" - info \n");
    printf(" - dump \n");
    printf(" - ptest \n"); 
    printf(" - wrestart \n"); 
    printf(" - fstart \n"); 
}


int main(int argc, char *argv[]) {
    printf("\n");
#ifdef DEBUG   
    int i;
    for (i = 0; i < argc; i++) 
    {
        printf("Argument %d is %s\n", i, argv[i]);
    }
    printf("\n"); 
#endif  
    if (argc == 1) {
        usage();
        return EXIT_FAILURE;
    }
    // the command 'port' never called for PCIe device. The port name environment variable is set in discovery subproject
    // if (!strcmp(argv[1], PORT_NAME)) {
    //     if (open_serial_port(argv[2]) == 0) {
    //         if (0 == SendCommand(INFO_CMD)) {     
    //             // means we got '@' - NEVER(!!!) comment out the following printf()
    //             // it seves as output for usb_port.sh script
    //             printf("Got @ acknowledge\n");
    //             return EXIT_SUCCESS;
    //         } else {
    //             return EXIT_FAILURE;
    //         }
    //     } else {
    //         return EXIT_FAILURE;
    //     }
    // }    
    if (open_serial_port(RetreivePortName()) == EXIT_SUCCESS) {
        // parse parameters here
        if (argc < 2) {
            printf("Wrong command format\n");
            usage();
            return EXIT_FAILURE;
        } 
        if (!strcmp(argv[1], INFO_CMD)){
           if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(INFO_CMD)) {
                read_info_from_port(GetSerialPort());
            } else {
                return EXIT_FAILURE;
            }
        }
        else if (!strcmp(argv[1], SET_CMD)) { 
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            KeyValue* settings;
            settings = GetSettingsFromFile();
            if (settings != NULL) {
                printf("Config file parsed\n");
                if (SendBatchCommand(settings) == 0) {
                    printf("Failed sending settings batch\n");
                    return EXIT_FAILURE;
                }
            } else {
                printf("Config file parsing problem\n");
                return EXIT_FAILURE;
            }
        }
        else if (!strcmp(argv[1], DUMP_CMD)) { 
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(DUMP_CMD)) {
                read_dump_from_port();
            } else {
                return EXIT_FAILURE;
            }
        }
        else if (!strcmp(argv[1], PROD_TEST)) {
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(PROD_TEST)) {
                read_test_data_from_port(GetSerialPort());
            } else {
                return EXIT_FAILURE;
            }
        }
        else if (!strcmp(argv[1], WARM_RESTART)) {
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(WARM_RESTART)) {
                printf("Command [%s] sent\n", WARM_RESTART);
            } else {
                printf("Command [%s] failed\n", WARM_RESTART);
                return EXIT_FAILURE;
            }
        }
        else if (!strcmp(argv[1], FIRST_START)) {
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(FIRST_START)) {
                printf("Command [%s] sent\n", FIRST_START);
            } else {
                printf("Command [%s] failed\n", FIRST_START);
                return EXIT_FAILURE;
            }
        }
        else {
            printf("Wrong (not supported) command \n");
            usage();
            return EXIT_FAILURE;
        }
    }
    printf("Success, exiting\n");
    return EXIT_SUCCESS;
}
