/*****************************************************************************
* Copyright:    (c) 2024 AMFELTEC Corp.
* ----------------------------------------------------------------------------
This file is part of AMFELTEC System Logger Software Package

# AMFELTEC System Logger Software Package 
# is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AMFELTEC System Logger Software Package
# is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <https://www.gnu.org/licenses/>

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <errno.h>

#include "outpututils.h"


#define STATUS_HEADER       "| Idx     |Temperature C |Humidity RH |Pressure hPA|\n"
#define STATUS_DIV_LINE     "----------------------------------------------------\n"

#define ACEL_HEADER         "| Idx     |   X coord  |   Y coord  |   Z coord  |\n"
#define ACEL_DIV_LINE       "--------------------------------------------------\n"

FILE* ST_FILE;
FILE* AC_FILE;
FILE* DUMP_FILE;

FILE* TDUMP_FILE;
FILE* FIN_LOG_FILE;

void zero_out_file_streams(void) {
    ST_FILE = NULL;
    AC_FILE = NULL;
    DUMP_FILE = NULL;
    TDUMP_FILE = NULL;
    FIN_LOG_FILE = NULL;
}


int write_status_header(){
    int retVal = 0;
    if (ST_FILE != NULL) {
        fprintf(ST_FILE, "%s", STATUS_DIV_LINE); 
        fprintf(ST_FILE, "%s", STATUS_HEADER);
        fprintf(ST_FILE, "%s", STATUS_DIV_LINE);
    } else {
        retVal = -1;
        perror("Status file not open, can't write header");
    }
    return retVal;
}

int write_accel_header(){
    int retVal = 0;
    if (ST_FILE != NULL) {
        fprintf(AC_FILE, "%s", ACEL_DIV_LINE); 
        fprintf(AC_FILE, "%s", ACEL_HEADER);
        fprintf(AC_FILE, "%s", ACEL_DIV_LINE);
    } else {
        retVal = -1;
        perror("Accel file not open, can't write header");
    }
    return retVal;
}

int add_line_to_status_file(char* line){
    int retVal = 0;
    if (ST_FILE != NULL) {
        fprintf(ST_FILE, "%s", line);
    } else {
        retVal = -1;
        perror("Status file not open, line not added");
    }
    return retVal;
}

int add_line_to_acel_file(char* line){
   int retVal = 0;
    if (AC_FILE != NULL) {
        fprintf(AC_FILE, "%s", line);
    } else {
        retVal = -1;
        perror("Status file not open, line not added");
    }
    return retVal;
}

int add_line_to_dump_file(char* line){
   int retVal = 0;
    if (DUMP_FILE != NULL) {
        fprintf(DUMP_FILE, "%s", line);
    } else {
        retVal = -1;
        perror("Dump file not open, line not added");
    }
    return retVal;
}

int write_to_dump_file(char *array) {
    int retVal = 0;
    if (TDUMP_FILE != NULL) {
        fprintf(TDUMP_FILE, "%s", array);
    } else {
        retVal = -1;
        perror("Temp Dump file not open, line not added");
    }
    return retVal;
}

int open_files(void) {
    int retVal = 0;

   if (mkdir(DATA_SUBDIRECTORY, 0777) == -1) {
        if (errno != EEXIST){
            perror("mkdir");
            return -1;
        }    
    }

    if (ST_FILE == NULL){
        ST_FILE = fopen(STATUS_FILE, "w");
        if (ST_FILE == NULL) {
            perror("Error opening status file for R/W");
            return -1;
        }
    }

    if (AC_FILE == NULL){
        AC_FILE = fopen(ACEL_FILE, "w");
        if (AC_FILE == NULL) {
            perror("Error opening accelerations file for R/W");
            return -1;
        }
    }

    if (DUMP_FILE == NULL){
        DUMP_FILE = fopen(FINAL_DUMP_FILE, "w");
        if (DUMP_FILE == NULL) {
            perror("Error opening dump file for R/W");
            return -1;
        }       
    } 
    if (TDUMP_FILE == NULL){
        TDUMP_FILE = fopen(TEMP_DUMP_FILE, "w");
        if (TDUMP_FILE == NULL) {
            perror("Error opening temp dump file for R/W");
            return -1;
        }       
    } 
    if (FIN_LOG_FILE == NULL){
        FIN_LOG_FILE = fopen(FINAL_LOG, "w");
        if (FIN_LOG_FILE == NULL) {
            perror("Error opening final log file for R/W");
            return -1;
        }       
    } 
    return retVal;
}


void close_files(void) {

    if (ST_FILE != NULL){
        fclose(ST_FILE);
    }

    if (AC_FILE != NULL){
        fclose(AC_FILE);
    }

    if (DUMP_FILE != NULL){
        fclose(DUMP_FILE);
    } 

    if (TDUMP_FILE != NULL){
        fclose(TDUMP_FILE);
    } 

    if (FIN_LOG_FILE != NULL){
        fclose(FIN_LOG_FILE);
    } 
}

