/*****************************************************************************
* Copyright:    (c) 2024 AMFELTEC Corp.
* ----------------------------------------------------------------------------
This file is part of AMFELTEC System Logger Software Package

# AMFELTEC System Logger Software Package 
# is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AMFELTEC System Logger Software Package
# is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <https://www.gnu.org/licenses/>

*****************************************************************************/

#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "utilities.h"

// below list of utilities used by dump parser


void removeHexCombination(char *input) {
    char *output = input;  // Output string starts at the same address as the input

    while (*input) {
        if (*input == 0x0A && *(input + 1) == 0x0A) {
            // Skip the second character of the hex combination, replace with space
            *output++ = ' ';
            input++;
        } 
        else {
            // Copy the character to the output string
            *output++ = *input;
        }
        // Move to the next character in the input string
        input++;
    }
    // Null-terminate the output string
    *output = '\0';
}

void replaceCombination(char *input) {
    // Find the first occurrence of "0x " in the input
    char *position = strstr(input, "0x ");
    //printf("Data buffer original:\n %s \n", input);
    // Loop until there are no more occurrences
    while (position != NULL) {
        // Replace "0x " with "0x"
        memmove(position + 2, position + 3, strlen(position + 3) + 1);
        // Move to the next occurrence
        position = strstr(position + 2, "0x ");
    }
}

int isValidHexString(char * str) {
    return (str[0] == '0' && (str[1] == 'x' && str[2] != '0') && strchr(str, ':') == NULL);
}


// Function to tokenize a string based on space and return an array of strings
char **tokenizeString(const char *input, int *numTokens) {
    char **tokens = NULL;
    *numTokens = 0;

    // Copy the input string to avoid modifying the original string
    char *inputCopy = strdup(input);

    // Tokenize the input string based on space
    char *token = strtok(inputCopy, " \r\n");
    while (token != NULL) {
        // Increase the number of tokens
        if (isValidHexString(token)) {
            (*numTokens)++;

            // Reallocate memory for the tokens array
            tokens = (char **)realloc(tokens, (*numTokens) * sizeof(char *));
            tokens[(*numTokens) - 1] = strdup(token);            
        }
        token = strtok(NULL, " \r\n");// Move to the next token
    }

    // Free the copy of the input string
    free(inputCopy);
    return tokens;
}


void reverseArray(char* arr[], int n) 
{ 
    char* temp; 
  
    // Move from begin and end. Keep 
    // swapping strings.  
    int j = n - 1; 
    for (int i = 0; i < j; i++) { 
        temp = arr[i]; 
        arr[i] = arr[j];  
        arr[j] = temp; 
        j--; 
    } 
} 

void printArray(char* arr[], int n) 
{ 
    for (int i = 0; i < n; i++) { 
        printf("%s ", arr[i]); 
    } 
} 

#ifdef DEBUG
void DumpHex(void* data, size_t size) {
	char ascii[17];
	size_t i, j;
	ascii[16] = '\0';
	for (i = 0; i < size; ++i) {
		printf("%02X ", ((unsigned char*)data)[i]);
		if (((unsigned char*)data)[i] >= ' ' && ((unsigned char*)data)[i] <= '~') {
			ascii[i % 16] = ((unsigned char*)data)[i];
		} else {
			ascii[i % 16] = '.';
		}
		if ((i+1) % 8 == 0 || i+1 == size) {
			printf(" ");
			if ((i+1) % 16 == 0) {
				printf("|  %s \n", ascii);
			} else if (i+1 == size) {
				ascii[(i+1) % 16] = '\0';
				if ((i+1) % 16 <= 8) {
					printf(" ");
				}
				for (j = (i+1) % 16; j < 16; ++j) {
					printf("   ");
				}
				printf("|  %s \n", ascii);
			}
		}
	}
}
#endif
