/*****************************************************************************
* Copyright:    (c) 2024 AMFELTEC Corp.
* ----------------------------------------------------------------------------
This file is part of AMFELTEC System Watchdog Software Package

# AMFELTEC System Watchdog Software Package 
# is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AMFELTEC System Watchdog Software Package
# is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <https://www.gnu.org/licenses/>

*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <sys/select.h>
#include <sys/ioctl.h>

#include "command.h"

int kbhit() {

    struct timeval tv;
    fd_set fds;

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    FD_ZERO(&fds);
    FD_SET(STDIN_FILENO, &fds);
    
    select(STDIN_FILENO + 1, &fds, NULL, NULL, &tv);
    return FD_ISSET(STDIN_FILENO, &fds);
}



int main(int argc, char *argv[]) {
    char input;
    
    // Disable buffering and echoing of characters in the terminal
    struct  termios oldt, newt;
    int     sysRet = 1;   
    tcgetattr(STDIN_FILENO, &oldt);
    newt = oldt;
    newt.c_lflag &= ~(ICANON | ECHO);
    tcsetattr(STDIN_FILENO, TCSANOW, &newt);

    //open_serial_port();
    // the following may be also called from any startup system file, or from application start as below
    sysRet = system("./wd_serial start");
    if (sysRet != 0) {
        printf("Watchdog failed to start, Exiting...\n");
        // Restore terminal settings before exiting
        tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
        return sysRet;
    }
    printf("Press 'q' to exit.\n");
    while (1) {
        printf("...\n");
        sleep(2);

        sysRet = system("./wd_serial refresh");
        if (sysRet != 0) {
            printf("Watchdog failed to refresh, Exiting...\n");
            tcsetattr(STDIN_FILENO, TCSANOW, &oldt); // Restore terminal settings before exiting
            return sysRet;
        }
        
        if (kbhit()) { // Check if 'q' is pressed
            input = getchar();
            if (input == 'q') {
                break; // Exit the loop if 'q' is pressed
            }
        }
    }

    tcsetattr(STDIN_FILENO, TCSANOW, &oldt); // Restore terminal settings before exiting
    printf("Exiting the program.\n");
    return 0;
}
