/*****************************************************************************
* Copyright:    (c) 2024 AMFELTEC Corp.
* ----------------------------------------------------------------------------
This file is part of AMFELTEC System Watchdog Software Package

# AMFELTEC System Watchdog Software Package 
# is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AMFELTEC System Watchdog Software Package
# is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <https://www.gnu.org/licenses/>

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "command.h"
#include "comport.h"


void usage(void) 
{
    printf("Usage :\n");
    printf("./wd_serial [command] <parameters>\n");
    printf("List of valid commands & parameters:\n");
    printf(" - info\n");
    printf(" - start\n");
    printf(" - start [0/1] [0/1] [01]\n");
    printf(" - stop\n");
    printf(" - refresh sec(optional)\n");
    printf(" - set\n");
    printf(" - set timer1 timer2 timer3 timer4 timer5 \n");
    printf("All time related parameters should fit in 1 byte, i.e. range [0 - 255]\n");
}

int main(int argc, char *argv[])
{
#ifdef DEBUG    
    int i;       
    printf("\n");
    for (i = 0; i < argc; i++) 
    {
        printf("Argument %d is %s\n", i, argv[i]);
    }
    printf("\n");   
#endif
    if (open_serial_port(RetreivePortName()) == EXIT_SUCCESS) {
        // parse parameters here
        if (argc < 2) {
            printf("Wrong command format\n");
            usage();
            return EXIT_FAILURE;
        }       
        if (!strcmp(argv[1], INFO_WD_CMD)) {
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(INFO_WD_CMD)) {
                read_info_from_port(GetSerialPort());
                return EXIT_SUCCESS;
            } else {
                return EXIT_FAILURE;
            }
        } else if (!strcmp(argv[1], START_WD_CMD)) {
            if (argc == 2 ) {
                if (0 == SendCommand(START_WD_CMD)) {
                    return EXIT_SUCCESS;
                } else {
                    return EXIT_FAILURE;
                }
            } else if (argc == 5) {
                if (0 == SendControlCommand(*argv[2], *argv[3], *argv[4])) {
                    return EXIT_SUCCESS;
                }  else {
                    return EXIT_FAILURE;
                }
            } else {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }

        } else if (!strcmp(argv[1], STOP_WD_CMD)) {
            if (argc != 2) {
                printf("Wrong command format\n");
                usage();
                return EXIT_FAILURE;
            }
            if (0 == SendCommand(STOP_WD_CMD)) {
                return EXIT_SUCCESS;
            } else {
                return EXIT_FAILURE;
            }
        } else if (!strcmp(argv[1], REFRESH_WD_CMD)) { 
            if (argc == 2) {
                SendRefresh(0);
            } else if (argc == 3) {
                SendRefresh(atoi(argv[2]));
            }
            return EXIT_SUCCESS;
        }  else if (!strcmp(argv[1], SET_WD_CMD)) {
            char timers[NUMBER_OF_TIMERS];
            if (argc == 2) {
                timers[0] = DEF_Wdog_INT;   
                timers[1] = DEF_RESET_INT; 
                timers[2] = DEF_PWR_OFF_INT; 
                timers[3] = DEF_PWR_WAIT_INT; 
                timers[4] = DEF_PWR_ON_INT; 
                if (0 == SendTimerCommand(timers)){
                    return EXIT_SUCCESS;
                } else {
                    return EXIT_FAILURE;
                }
            } else if (argc == 7) {
                int i;
                for (i = 0; i < NUMBER_OF_TIMERS; i++) {
                    timers[i] = *argv[i + 2];
                }
                if (0 == SendTimerCommand(timers)){
                    return EXIT_SUCCESS;
                } else {
                    return EXIT_FAILURE;
                }
            } else {
                printf("Wrong set command parameters\n");
                usage();
                return EXIT_FAILURE;
            }
        
 
        } else {
            printf("Wrong command\n");
            usage();
            return EXIT_FAILURE;
        }
    } else {
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
